@@ -4,6 +4,7 @@ import logging
 import sys
 from decimal import Decimal
 
+import reversion
 from deepmerge import Merger
 from graphql.language.printer import print_ast
 import graphene
@@ -11,7 +12,7 @@ from django.contrib.gis.db.models import GeometryField, OneToOneField, ManyToMan
     GeometryCollectionField
 from graphql import parse
 
-from rescape_python_helpers import ramda as R
+from rescape_python_helpers import ramda as R, memoize
 from django.contrib.postgres.fields import JSONField
 from django.db.models import AutoField, CharField, BooleanField, BigAutoField, DecimalField, \
     DateTimeField, DateField, BinaryField, TimeField, FloatField, EmailField, UUIDField, TextField, IntegerField, \
@@ -22,7 +23,6 @@ from inflection import camelize
 from rescape_python_helpers.functional.ramda import to_dict_deep, flatten_dct, to_pairs, flatten_dct_until
 
 from .graphene_helpers import dump_graphql_keys, dump_graphql_data_object
-from .memoize import memoize
 
 logger = logging.getLogger('rescape_graphene')
 from django.conf import settings
@@ -960,3 +960,44 @@ def merge_data_fields_on_update(data_fields, existing_instance, data):
             ], ["override"], ["override"])
         )
     )
+
+
+def delete_if_marked_for_delete(model_cls, grapene_upsert_class, upsert_field_name, model_data):
+    """
+    Delete functionality for graphene models whose django_model mixes in  safe-delete
+    This looks for a deleted=datetime field and calles manage.delete on the instance
+    matching model_data.id if deleted is not null
+    :param model_cls: The Django model class
+    :param grapene_upsert_class: The Graphene upsert class
+    :param upsert_field_name: The named argument to pass when instantiating the upsert class, e.g. 'region' for Region
+    :param model_data:  The model data. Must have non-null id and deleted fields to cause a deletion
+    :return:
+    """
+
+    if R.all_satisfy(lambda prop: R.prop_or(False, prop, model_data), ['id', 'deleted']):
+        # Existing objects with a nonnull deleted datetime are deleted
+        instances = model_cls.objects.filter(
+            id=R.prop('id', model_data)
+        )
+        instances.delete()
+        instance = model_cls.objects.deleted_only().get(**R.pick(['id'], model_data))
+        if not instance:
+            raise Exception(f"Failed to delete {R.prop('id', instance)}")
+        return grapene_upsert_class(**{upsert_field_name: instance})
+    return None
+
+
+def update_or_create_with_revision(model_class, update_or_create_values):
+    """
+        Perform update or create with the given model_class and update_or_create_values
+        where the model_class must be registered with django-reversion. The update or create
+        saves a new revision
+    :param model_class:
+    :param update_or_create_values:
+    :return: The tuple from update_or_create
+    """
+
+    # Declare a revision block.
+    with reversion.create_revision():
+        return model_class.objects.update_or_create(**update_or_create_values)
+
